<?php

namespace FiloBlu\Storelocator\Model\Plugin\Rma;

use Magento\Rma\Model\Rma as MagentoRma;
use Magento\Store\Model\Store;
use Magento\Sales\Model\Order\Address;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Rma\Api\RmaAttributesManagementInterface;
use Magento\Framework\App\RequestInterface;
use Psr\Log\LoggerInterface;

class Rma
{
    /**
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $_helper;

    /**
     * @var \FiloBlu\Storelocator\Model\StoresFactory
     */
    protected $_storesFactory;
    /**
     * @var \Magento\Rma\Helper\Data
     */
    protected $_rmaHelper;
    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $_inlineTranslation;
    /**
     * @var \Magento\Payment\Helper\Data
     */
    protected $_paymentHelper;
    /**
     * @var Address\Renderer
     */
    protected $_addressRenderer;
    /**
     * @var RequestInterface
     */
    private $request;
    /**
     * @var \Magento\Rma\Model\RmaFactory
     */
    private $rmaFactory;
    /**
     * @var \Magento\Rma\Model\ResourceModel\Rma
     */
    private $rma;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Rma constructor.
     * @param \FiloBlu\Storelocator\Helper\Data $helper
     * @param \FiloBlu\Storelocator\Model\StoresFactory $storesFactory
     * @param \Magento\Rma\Helper\Data $rmaHelper
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Payment\Helper\Data $paymentHelper
     * @param Address\Renderer $addressRenderer
     * @param RequestInterface $request
     * @param \Magento\Rma\Model\RmaFactory $rmaFactory
     * @param \Magento\Rma\Model\ResourceModel\Rma $rma
     * @param LoggerInterface $logger
     */
    public function __construct(
        \FiloBlu\Storelocator\Helper\Data $helper,
        \FiloBlu\Storelocator\Model\StoresFactory $storesFactory,
        \Magento\Rma\Helper\Data $rmaHelper,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Magento\Payment\Helper\Data $paymentHelper,
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer,
        RequestInterface $request,
        \Magento\Rma\Model\RmaFactory $rmaFactory,
        \Magento\Rma\Model\ResourceModel\Rma $rma,
        LoggerInterface $logger
    )
    {
        $this->_helper = $helper;
        $this->_storesFactory = $storesFactory;
        $this->_rmaHelper = $rmaHelper;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->_inlineTranslation = $inlineTranslation;
        $this->_paymentHelper = $paymentHelper;
        $this->_addressRenderer = $addressRenderer;
        $this->request = $request;
        $this->rmaFactory = $rmaFactory;
        $this->rma = $rma;
        $this->logger = $logger;
    }

    public function beforeSaveRma(\Magento\Rma\Model\Rma $subject, $data) {

        if (!$this->_helper->getReturnToStoreEnabled()) {
            return [$data];
        }

        if(isset($data['rma_returntostore_store_id']) && $data['rma_returntostore_store_id'] > 0) {
            $subject->setData('return_to_store_id', $data['rma_returntostore_store_id']);
            $subject->setData('return_to_store_email', $data['rma_returntostore_email']);
            $subject->setData('return_to_store_hash', (md5(time())));
        }

        return [$data];
    }

    /**
     * @param MagentoRma $subject
     * @param $result
     * @return array|null
     */
    public function afterSaveRma(MagentoRma $subject, $result)
    {

        if (!$this->_helper->getReturnToStoreEnabled()) {
            return $result;
        }

        try {
            if (!$result) {
                return $result;
            }

            $this->sendEmails($subject);

            return $result;
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return $result;
        } catch (\Throwable $t) {
            $this->logger->critical($t->getMessage());
            return $result;
        }
    }

    public function sendEmails($rma) {

        $returnInStoreEnabled = $this->_helper->getReturnToStoreEnabled();
        $returnToStoreId = (int)$rma->getReturnToStoreId();

        if (!$returnInStoreEnabled) {
            return $rma;
        }

        if (!$rma->getReturnToStoreId()) {
            return $rma;
        }

        if ($returnToStoreId) {
            if ($rma->getStatus() == 'pending') {
                try {
                    if ($rma->getData('return_to_store_received_date') == null) {
                        $this->_helper->sendRmaMail($rma);
                    }
                } catch (\Exception $ex) {

                }
            }
        } else {
            try {
                if ($rma->getData('return_to_store_received_date') == null) {
                    //$this->_helper->sendRmaMail($rma);
                }
            } catch (\Exception $ex) {

            }
        }
    }

}
